#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::Constants
# state        public
#----------------------------------------------------------------------------------------
# description  This is the place for common DHL Intraship values and constants.
## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
#========================================================================================
package DE_EPAGES::DHLIntraship::API::Constants;
use base Exporter;

use strict;

our @EXPORT_OK = qw (
    $DHLINTRASHIPSERVICEPATH
    $SERVICESFOLDERNAME
    $DHLVERSENDEN_SHIPMENT_RELEASE
    $SHIPPING_WSDL
    $LOCATION_WSDL
    $DEFAULTSELECTNAME
    @OPERATIONS
    @SERVICES
    @PORTS
    $EKP_LENGHT
    $PRODUCTCODE_LENGHT
    %MAPPING_PRODUCTCODE_PRODUCTNAME
    %DHLINTRASHIP_LABELSTATUS
    $DHLINTRASHIP_WRONG_POSTNUMMER_MESSAGE
    %MAPPING_PRODUCTCODE_FULLPRODUCTNAME
    $STATUS_CODE_OK
    $NUMBER_OF_PACKSTATIONS
    %SHIPPING_LABEL_STATUS
    %SOAP_REQUEST_PORT
    %REALM
    $CURRENCY
    $GERMAN_LOCALE
    %ADDRESS_LENGTHS
    %PACKSTATION_ADDRESS_LENGTHS
    %ADDRESS_LENGTH_INTERVALS
    $DHLINTRASHIP_PATH_WSDL
    $XSD_STANDORTE_FILE
    $XSD_CISBASE_FILE
    $XSD_ISBASE_FILE
    $MAX_ATTRIBUTE_LENGTH
    $GERMANY_COUNTRY_ID
    @VISIBLE_SHIPPING_ADDRESS_FIELDS
    @MANDATORY_SHIPPING_ADDRESS_FIELDS
    $DEFAULT_HIGHER_INSURANCE_AMOUNT
    $DEFAULT_BULKFREIGHT_TYPE
    %EPAGES_SERVICES_TO_DHLSERVICES
    %DHLVERSENDEN_STATUS_CODES
    $GET_MANIFEST_OPERATION_IS_VALID
    $GET_MANIFEST_OPERATION_IS_UNVALID
    $GET_MANIFEST_OPERATION_IS_UNKNOWN
    DELIVERY_DATE_IN_THE_PAST_WARNING_STRING
    DELIVERY_DATE_IN_THE_PAST_WARNING_KEYWORD
    MANDATORY_WARNING_KEYWORD
    MANDATORY_WARNING_STRING
    TOO_LONG_WARNING_KEYWORD
    TOO_LONG_WARNING_STRING
    TOO_SHORT_WARNING_KEYWORD
    TOO_SHORT_WARNING_STRING
    TOTAL_WEIGHT_TOO_HIGH_WARNING_STRING
    TOTAL_WEIGHT_TOO_HIGH_WARNING_KEYWORD
    TOTAL_WEIGHT_TOO_LOW_WARNING_STRING
    TOTAL_WEIGHT_TOO_LOW_WARNING_KEYWORD
    MAX_WEIGHT
    MIN_WEIGHT
);

our $DHLINTRASHIPSERVICEPATH = '/DHLIntrashipServices';
our $SERVICESFOLDERNAME = 'DHLIntrashipServices';
our $DHLVERSENDEN_SHIPMENT_RELEASE = '3.3';
our $SHIPPING_WSDL = 'geschaeftskundenversand-api-3.3.0.wsdl';
our $LOCATION_WSDL = 'standortsuche-api-1.1.wsdl';
our $DEFAULTSELECTNAME = '-none-';
our $EKP_LENGHT = 10;
our $PRODUCTCODE_LENGHT = 2;
our $STATUS_CODE_OK = 0;
our $NUMBER_OF_PACKSTATIONS = 5;
our $CURRENCY = 'EUR';
our $GERMAN_LOCALE = 'DE';
our $DHLINTRASHIP_PATH_WSDL = '/DE_EPAGES/DHLIntraship/API/SOAP/WSDL/';
our $XSD_STANDORTE_FILE = 'standorte.xsd';
our $XSD_CISBASE_FILE = 'geschaeftskundenversand-api-3.3.0-schema-cis_base.xsd';
our $XSD_ISBASE_FILE = 'geschaeftskundenversand-api-3.3.0-schema-bcs_base.xsd';
our $GERMANY_COUNTRY_ID = 276;

our %ADDRESS_LENGTHS = (
    'city'          => 50,
    'packstationId' => 3,
    'street'        => 30,
    'zipcode'       => 5,
);

our %PACKSTATION_ADDRESS_LENGTHS = (
    'city'          => 50,
    'packstationId' => 3,
    'street'        => 30,
    'zipcode'       => 5,
    'name'          => 50,
);

our %ADDRESS_LENGTH_INTERVALS = (
    'accountNumber'         => { 'required' => 1, 'max' => 14, 'min' => 14 },
    'addressAddition'       => { 'required' => 0, 'max' => 35, 'min' => 1 },
    'city'                  => { 'required' => 1, 'max' => 50, 'min' => 1 },
    'country'               => { 'required' => 0, 'max' => 30, 'min' => 1 },
    'email'                  => { 'required' => 0, 'max' => 70, 'min' => 1 },
    'name1'                 => { 'required' => 1, 'max' => 35, 'min' => 1 },
    'name2'                 => { 'required' => 0, 'max' => 35, 'min' => 1 },
    'packstationNumber'     => { 'required' => 1, 'max' => 3, 'min' => 3 },
    'phone'                 => { 'required' => 0, 'max' => 35, 'min' => 1 },
    'postNumber'            => { 'required' => 0, 'max' => 10, 'min' => 1 },
    'recipientEmailAddress' => { 'required' => 1, 'max' => 70, 'min' => 1 },
    'sequenceNumber'        => { 'required' => 1, 'max' => 35, 'min' => 1 },
    'state'                 => { 'required' => 0, 'max' => 30, 'min' => 1 },
    'streetName'            => { 'required' => 1, 'max' => 50, 'min' => 1 },
    # AD-10236: documentation specifies 10 characters as maximum for street number,
    # but DHL services report an error if longer than 5.
    'streetNumber'          => { 'required' => 1, 'max' => 10, 'min' => 1 },
    'zip'                   => { 'required' => 1, 'max' => 17, 'min' => 1 },
);

our %MAPPING_PRODUCTCODE_PRODUCTNAME = (
            '01' => 'V01PAK',
            '53' => 'V53WPAK',
            '54' => 'V54EPAK',
);

our %MAPPING_PRODUCTCODE_FULLPRODUCTNAME = (
            '01' => 'Paket',
            '53' => 'Weltpaket',
            '54' => 'Europaket',
);

our @OPERATIONS = qw ( createShipmentTD
                        createShipmentDD
                        deleteShipmentTD
                        deleteShipmentDD
                        doManifestTD
                        doManifestDD
                        getLabelTD
                        getLabelDD
                        bookPickup
                        cancelPickup
                        getVersion
                        getExportDocTD
                        getExportDocDD);

our @SERVICES = qw ( GVAPI_2_0_de WebServiceImplService WebServiceImplServiceSandbox );

our @PORTS = qw ( GKVAPISOAP11port0 WebServiceImplPort );

our %SOAP_REQUEST_PORT = (
    'NO_SSL' => 80,
    'SSL'    => 443
);

our %DHLINTRASHIP_LABELSTATUS = (
    'NEW'                               => 0,
    'CREATED'                           => 1,
    'MANIFESTED'                        => 2,
    'ERROR'                             => 3,
    'WRONG_POSTNUMBER'                  => 4,
);

our $DHLINTRASHIP_WRONG_POSTNUMMER_MESSAGE = 'Die verwendete Postnummer ist nicht';

our %SHIPPING_LABEL_STATUS = (
            'ALL_OK' => 'ALL_EXPORTED',
            'ALL_KO' => 'ALL_EXPORTED_FAILED',
            'SOME_OK' => 'SOME_EXPORTED_FAILED',
            'WRONG_CREDENTIALS' => 'WRONG_CREDENTIALS',
);

our %REALM = (
    'TEST' => 'CIG Sandbox Services',
    'LIVE' => 'CIG Production Services',
);

our $MAX_ATTRIBUTE_LENGTH = {
    'careOfName'      => 50,
    'Company'         => 30,
    'roomNumber'      => 10,
};

our @VISIBLE_SHIPPING_ADDRESS_FIELDS    = qw (FirstName LastName City Street Zipcode EMail);
our @MANDATORY_SHIPPING_ADDRESS_FIELDS  = qw (FirstName LastName City Street Zipcode EMail);

our $DEFAULT_HIGHER_INSURANCE_AMOUNT = 2500;
our $DEFAULT_BULKFREIGHT_TYPE = 'L';

our %EPAGES_SERVICES_TO_DHLSERVICES = (
    'Transportversicherung'             => {
                                                'AdditionalInsurance' =>    {
                                                                                'active' => 1,
                                                                                'insuranceAmount' => 2500
                                                                            }
                                           },
    'Sperrgut'                           => {
                                                'BulkyGoods' => {
                                                                    'active' => 1
                                                                }
                                            },
    'Eigenhndig'                        => {
                                                'Personally' => {
                                                                    'active' => 1
                                                                }
                                            },
);

our %DHLVERSENDEN_STATUS_CODES = (
        'valid_login' => [
                0, # valid credentials and shipping label data is found for the requested date
                2000 # valid credentials but no shipping label data is found for the requested date
        ],
        'unvalid_login' => [
                112, # login failed because of expired password
                1001 # login failed because unvalid username and or signature used
        ]
);

our $GET_MANIFEST_OPERATION_IS_VALID = 'VALID';
our $GET_MANIFEST_OPERATION_IS_UNVALID = 'UNVALID';
our $GET_MANIFEST_OPERATION_IS_UNKNOWN = 'UNKNOWN';

use constant DELIVERY_DATE_IN_THE_PAST_WARNING_STRING => 'delivery date is in the past';
use constant DELIVERY_DATE_IN_THE_PAST_WARNING_KEYWORD => 'ImpossibleDeliveryDate';
use constant MANDATORY_WARNING_KEYWORD => 'AttributeMandatoryNotSetWarning';
use constant MANDATORY_WARNING_STRING => 'is mandatory but not set';
use constant TOO_LONG_WARNING_KEYWORD => 'AttributeTooLongWarning';
use constant TOO_LONG_WARNING_STRING => 'is too long';
use constant TOO_SHORT_WARNING_KEYWORD => 'AttributeTooShortWarning';
use constant TOO_SHORT_WARNING_STRING => 'is too short';
use constant TOTAL_WEIGHT_TOO_HIGH_WARNING_STRING => 'total weight is too high';
use constant TOTAL_WEIGHT_TOO_HIGH_WARNING_KEYWORD => 'TotalWeightTooHigh';
use constant TOTAL_WEIGHT_TOO_LOW_WARNING_STRING => 'total weight is too low';
use constant TOTAL_WEIGHT_TOO_LOW_WARNING_KEYWORD => 'TotalWeightTooLow';

use constant MAX_WEIGHT => 31.5;
use constant MIN_WEIGHT => 0.1;
## use critic (ValuesAndExpressions::ProhibitMagicNumbers)
