#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Communication
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Communication module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Communication;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Communication;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

use constant TEST_ADDRESS_PHONE_NUMBER => '935643245';
use constant TEST_ADDRESS_PHONE_NUMBER_TOO_LONG => '+40-00-123-456.6758-0344-22233356666';
use constant TEST_ADDRESS_EMAIL => 'test@epages.com';
use constant TEST_ADDRESS_EMAIL_TOO_LONG =>
        'email_name_that_exceeds_the_maximum_characters_limit@subdomain_too_long.epages.com';


#========================================================================================
sub test {
    my $self = shift;

    $self->communication_assemble_positive_test();
    $self->communication_assemble_address_fields_too_long_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub communication_assemble_positive_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress();
    my $hCommunication =
        DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Communication->assemble($mockedAddress);

    is_deeply($hCommunication->{'nodeData'},
        { 'Communication' => {
              'phone' => TEST_ADDRESS_PHONE_NUMBER,
              'email' => TEST_ADDRESS_EMAIL,
          }
        },
        "$SubTestName - Communication hash assembled"
    );

    is_deeply($hCommunication->{'nodeWarnings'}, [], "$SubTestName - warnings array for Communication is empty");
}

sub communication_assemble_address_fields_too_long_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress(
        { 'Phone' => TEST_ADDRESS_PHONE_NUMBER_TOO_LONG, 'EMail' => TEST_ADDRESS_EMAIL_TOO_LONG }
    );
    my $hCommunication =
        DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Communication->assemble($mockedAddress);
    my $PhoneResult = $hCommunication->{'nodeData'}->{'Communication'}->{'phone'};
    my $EmailResult = $hCommunication->{'nodeData'}->{'Communication'}->{'email'};

    ok(length($PhoneResult) > 0 &&  length($PhoneResult) < length(TEST_ADDRESS_PHONE_NUMBER_TOO_LONG),
        "$SubTestName - Phone has been trimmed because it was too long"
    );

    ok(length($EmailResult) > 0 &&  length($EmailResult) < length(TEST_ADDRESS_EMAIL_TOO_LONG),
        "$SubTestName - Email name has been trimmed because it was too long"
    );

    is_deeply($hCommunication->{'nodeWarnings'},
        [
             { 'phone' => 'StringTooLong' },
             { 'email' => 'StringTooLong' }
        ],
        "$SubTestName - warnings array for Communication holds phone and email messages about being too long"
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createAddress {
    my $self = shift;
    my ($hAddress) = @_;

    $hAddress = {} unless defined($hAddress);

    my $hParameterList = {
        'Phone' => $hAddress->{'Phone'} // TEST_ADDRESS_PHONE_NUMBER,
        'EMail' => $hAddress->{'EMail'} // TEST_ADDRESS_EMAIL,
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            $hParameterList
    );

    return $MockAddress->getMockObject();
}

__PACKAGE__->runScript;
