#========================================================================================
# package      DE_EPAGES::ExternalShipping::t::ShippingFulfillmentType
# state        public
#----------------------------------------------------------------------------------------
# description  ShippingFulfillmentType functionality
#========================================================================================
package DE_EPAGES::ExternalShipping::t::ShippingFulfillmentType;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify::Sut;

use Test::Mockify::Matcher qw (String );
use Test::More;



use strict;



sub test {
    my $self = shift;

    $self->test_isDeprecated();
    $self->test_isDeprecatedUnity();
    $self->test_NotDeprecated();
    $self->test_NotDeprecatedUnity();

    done_testing();
}

sub test_isDeprecated {
    my ($self) = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hShopParams = {
        IsUnityRegistered => 0
    };
    my $Shop = $self->_createMockedShop($hShopParams);

    my $SUTParams = {
        IsDeprecated      => 1,
        IsDeprecatedUnity => 0
    };
    my $SUT = $self->_createSUT($SUTParams);

    my $Response = $SUT->isDeprecated($Shop);

    is($Response, 1, "$SubTestName - True, if BaseShop and Shipping deprecated for Base" );
}

sub test_isDeprecatedUnity {
    my ($self) = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hShopParams = {
        IsUnityRegistered => 1
    };
    my $Shop = $self->_createMockedShop($hShopParams);

    my $SUTParams = {
        IsDeprecated      => 0,
        IsDeprecatedUnity => 1
    };
    my $SUT = $self->_createSUT($SUTParams);

    my $Response = $SUT->isDeprecated($Shop);

    is($Response, 1, "$SubTestName - True, if NowShop and Shipping deprecated for Unity" );
}

sub test_NotDeprecated {
    my ($self) = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hShopParams = {
        IsUnityRegistered => 0
    };
    my $Shop = $self->_createMockedShop($hShopParams);

    my $SUTParams = {
        IsDeprecated      => 0,
        IsDeprecatedUnity => 1
    };
    my $SUT = $self->_createSUT($SUTParams);

    my $Response = $SUT->isDeprecated($Shop);

    is($Response, 0, "$SubTestName - False, if BaseShop and Shipping not deprecated for Base" );
}

sub test_NotDeprecatedUnity {
    my ($self) = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hShopParams = {
        IsUnityRegistered => 1
    };
    my $Shop = $self->_createMockedShop($hShopParams);

    my $SUTParams = {
        IsDeprecated      => 1,
        IsDeprecatedUnity => 0
    };
    my $SUT = $self->_createSUT($SUTParams);

    my $Response = $SUT->isDeprecated($Shop);

    is($Response, 0, "$SubTestName - False, if NowShop and Shipping not deprecated for Unity" );

}
sub _createSUT {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        DeprecationDate      => $self->_createDeprecationDate($hParams->{'IsDeprecated'}),
        DeprecationDateUnity => $self->_createDeprecationDate($hParams->{'IsDeprecatedUnity'}),
    };
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType', [ $hAttributes ]);
    $MockifySut->mock('get')
        ->when(String('DeprecationDateUnity'))
        ->thenReturn($hAttributes->{DeprecationDateUnity});

    $MockifySut->mock('get')
        ->when(String('DeprecationDate'))
        ->thenReturn($hAttributes->{DeprecationDate});
    return $MockifySut->getMockObject();
}

sub _createMockedShop {
    my $self = shift;
    my ($hParams) = @_;
    my $hAttributes = {
        'IsUnityRegistered' => $hParams->{'IsUnityRegistered'},
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    return $MockifyShop->getMockObject();
}


sub _createDeprecationDate {
    my $self = shift;
    my ($IsDeprecated) = @_;

    if ($IsDeprecated) {
        return DateTime->new(
            year      => 2000,
            month     => 5,
            day       => 10,
            hour      => 15,
            minute    => 15,
            time_zone => 'America/Los_Angeles',
        );
    } else {
        DateTime->new(
            year      => 2030,
            month     => 5,
            day       => 10,
            hour      => 15,
            minute    => 15,
            time_zone => 'America/Los_Angeles',
        );
    }
}

__PACKAGE__->runScript();
