#========================================================================================
# package      DE_EPAGES::ProductPortal::t::unit::UI_ProductPortal
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for UI::ProductPortal
#========================================================================================
package DE_EPAGES::ProductPortal::t::unit::UI_ProductPortal;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Core::API::PerlTools qw (GetCurrentDateTime SubtractDuration);
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetCallCount GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Mockify::Matcher qw ( HashRef Object String ArrayRef );
use Test::Mockify::Sut;
use Test::More;
use JSON;
use utf8;
use Encode;

use strict;

use constant PORTAL_SITE_ID => '1222';
use constant EXISTING_PORTAL_CONFIG_ID => '1234';
use constant ACTIVE_PORTAL_CONFIG_ID => '5678';


sub test {
    my $self = shift;
    $self->test_UpdatePortalConfig();
    $self->test_UpdatePortalConfig_IsOnlyGoogle_delete();
    $self->test_UpdatePortalConfig_IsOnlyGoogle_add();
    $self->test_UpdatePortalConfig_IsOnlyAmazon_delete();
    $self->test_UpdatePortalConfig_IsOnlyAmazon_add();
    $self->test_UpdatePortalConfig_add_PortalConfig();
    $self->test_UpdatePortalConfig_add_PortalConfig_deprecated();
    $self->test_UpdatePortalConfig_add_and_delete_PortalConfigs();
    $self->test_UpdatePortalConfig_delete_PortalConfigs_confirm_google_and_amazon_are_not_affected();
    $self->test_UpdatePortalConfig_add_and_delete_PortalConfigs_deprecated();
    $self->test_UpdatePortalConfig_delete_PortalConfig();
    $self->test_UpdatePortalConfig_delete_PortalConfig_deprecated();
    $self->test_UpdatePortalConfig_case_delete_Confirmation_generic_portal();
    $self->test_UpdatePortalConfig_case_delete_Confirmation_only_amazon_with_amazon_portal();
    $self->test_UpdatePortalConfig_case_delete_Confirmation_only_amazon_no_amazon_portal();
    $self->test_UpdatePortalConfig_case_delete_Confirmation_only_amazon_with_googleBase_portal();
    $self->test_UpdatePortalConfig_case_delete_Confirmation_only_amazon_no_googleBase_portal();
    $self->test_GeneratePortalConfigProductPrettyUnityUrls_no_unity_shop();
    $self->test_GeneratePortalConfigProductPrettyUnityUrls_is_unity_shop_no_portalconfig_completed();
    $self->test_GeneratePortalConfigProductPrettyUnityUrls_is_unity_shop_is_portalconfig_completed();
    done_testing();
}

sub test_UpdatePortalConfig {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => undef,
        OnlyAmazon             => undef,
        AddPortalConfig        => undef,
        DeletePortalConfig     => undef,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => ['Amazon', 'myPortal'],
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);
    #is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation was not called.");
    #my $portalConfigIDS = $Servlet->vars('PortalConfigIDs') //[];
    #is(scalar @$portalConfigIDS, 1, "$SubTestName - There are portal configIDs in Servlet.");
    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NO portal configs to delete.");
}

sub test_UpdatePortalConfig_IsOnlyGoogle_delete {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => 1,
        OnlyAmazon             => undef,
        AddPortalConfig        => undef,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => ['Amazon', 'myPortal'],
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation was not called.");

    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NO portal configs to delete.");
}

sub test_UpdatePortalConfig_IsOnlyGoogle_add {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => 1,
        OnlyAmazon             => undef,
        AddPortalConfig        => 1,
        DeletePortalConfig     => undef,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => ['Amazon', 'myPortal', 'GoogleBase'],
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NO portal configs to delete.");
    is($Servlet->vars('PortalConfigIDs'), undef, "$SubTestName - There are NO portal configIDs in Servlet.");
    is(GetCallCount($SUT, '_createPortalConfig'), 1, "$SubTestName - Portal config creation was called.");

}

sub test_UpdatePortalConfig_IsOnlyAmazon_delete {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => undef,
        OnlyAmazon             => 1,
        AddPortalConfig        => undef,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => ['GoogleBase', 'myPortal'],
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation was not called.");

    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NO portal configs to delete, only Amazon Should be deleted.");
}

sub test_UpdatePortalConfig_IsOnlyAmazon_add {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => undef,
        OnlyAmazon             => 1,
        AddPortalConfig        => 1,
        DeletePortalConfig     => undef,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => ['GoogleBase', 'myPortal', 'GoogleBase'],
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NO portal configs to delete.");
    is($Servlet->vars('PortalConfigIDs'), undef, "$SubTestName - There are NO portal configIDs in Servlet.");
    is(GetCallCount($SUT, '_createPortalConfig'), 1, "$SubTestName - Portal config creation was called.");

}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_add_PortalConfig {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        AddPortalConfig        => 1,
        DeletePortalConfig     => undef,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NOT portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 1, "$SubTestName - Portal config creation was called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_add_PortalConfig_deprecated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        AddPortalConfig        => 1,
        DeletePortalConfig     => undef,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NOT portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation method was NOT called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_add_and_delete_PortalConfigs {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        AddPortalConfig        => 1,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), 1, "$SubTestName - There ARE portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 1, "$SubTestName - Portal config creation method WAS called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_delete_PortalConfigs_confirm_google_and_amazon_are_not_affected {

    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        AddPortalConfig        => 0,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => ['Amazon', 'GoogleBase', 'GoogleSmartShopping'],
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are no portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation method was NOT called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_add_and_delete_PortalConfigs_deprecated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        AddPortalConfig        => 1,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => undef,
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), 1, "$SubTestName - There ARE portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation method does NOT gets called (deprecated).");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_delete_PortalConfig {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        AddPortalConfig        => 0,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => undef,
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), 1, "$SubTestName - There ARE portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation method does NOT gets called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_delete_PortalConfig_deprecated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        AddPortalConfig        => 0,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases   => undef,
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), 1, "$SubTestName - There ARE portal configs to delete (deprecation does not prevent deletion).");
}

sub test_UpdatePortalConfig_case_delete_Confirmation_generic_portal() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        AddPortalConfig        => 0,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases  => undef,
        ACCEPT                 => 1
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - Preparation for the delete confirmation dialogue DELETE flag was true in previous response.");
    ok( WasCalled($SUT , '_removeExportFile'), "$SubTestName - _removeExportFile was called");

}

sub test_UpdatePortalConfig_case_delete_Confirmation_only_amazon_with_amazon_portal() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => undef,
        OnlyAmazon             => 1,
        AddPortalConfig        => 0,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases  => ['Amazon'],
        ACCEPT                 => 1
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - Preparation for the delete confirmation dialogue DELETE flag was true in previous response.");
    ok( WasCalled($SUT , '_removeExportFile'), "$SubTestName - _removeExportFile was called.");
}

sub test_UpdatePortalConfig_case_delete_Confirmation_only_amazon_no_amazon_portal() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => undef,
        OnlyAmazon             => 1,
        AddPortalConfig        => 0,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases  => ['GoogleBase'],
        ACCEPT                 => 1
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - Preparation for the delete confirmation dialogue DELETE flag was true in previous response.");
    ok( !WasCalled($SUT , '_removeExportFile'), "$SubTestName - _removeExportFile was called.");
}

sub test_UpdatePortalConfig_case_delete_Confirmation_only_amazon_with_googleBase_portal() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => 1,
        OnlyAmazon             => undef,
        AddPortalConfig        => 0,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 0,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases  => ['GoogleBase'],
        ACCEPT                 => 1
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - Preparation for the delete confirmation dialogue DELETE flag was true in previous response.");
    ok( WasCalled($SUT , '_removeExportFile'), "$SubTestName - _removeExportFile was called.");
}

sub test_UpdatePortalConfig_case_delete_Confirmation_only_amazon_no_googleBase_portal() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => 1,
        OnlyAmazon             => undef,
        AddPortalConfig        => 0,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases  => ['Amazon'],
        ACCEPT                 => 1
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - Preparation for the delete confirmation dialogue DELETE flag was true in previous response.");
    ok( !WasCalled($SUT , '_removeExportFile'), "$SubTestName - _removeExportFile was called.");
}

sub test_UpdatePortalConfig_case_delete_Confirmation_googleSmartShopping_not_affected() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupUpdatePortalConfig({
        OnlyGoogleShopping     => undef,
        OnlyAmazon             => undef,
        AddPortalConfig        => undef,
        DeletePortalConfig     => undef,
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => 1,
        ExportAllProducts      => undef,
        ExistingPortalAliases  => ['GoogleSmartShopping'],
        ACCEPT                 => 1
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->UpdatePortalConfig($Servlet);

    #THEN
    is($Servlet->vars('DELETE'), undef, "$SubTestName - Preparation for the delete confirmation dialogue DELETE flag was true in previous response.");
    ok( !WasCalled($SUT , '_removeExportFile'), "$SubTestName - _removeExportFile was called.");
}

#----------------------------------------------------------------------------------------
sub test_GeneratePortalConfigProductPrettyUnityUrls_no_unity_shop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupPrettyURLS({
        AddPortalConfig        => undef,
        DeletePortalConfig     => 1,
        IsUnityRegistered      => undef,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => undef,
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->GeneratePortalConfigProductPrettyUnityUrls($Servlet);

    #THEN
    is(GetCallCount($SUT, '_GetAndSaveProductPrettyUnityUrls'), 0, "$SubTestName - Portal config pretty url request generation was NOT called.");
}

#----------------------------------------------------------------------------------------
sub test_GeneratePortalConfigProductPrettyUnityUrls_is_unity_shop_no_portalconfig_completed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupPrettyURLS({});
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};

    #WHEN
    $SUT->GeneratePortalConfigProductPrettyUnityUrls($Servlet);

    #THEN
    is(GetCallCount($SUT, '_GetAndSaveProductPrettyUnityUrls'), 0, "$SubTestName - Portal config pretty url request generation does NOT gets called.");
}

#----------------------------------------------------------------------------------------
sub test_GeneratePortalConfigProductPrettyUnityUrls_is_unity_shop_is_portalconfig_completed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    #GIVEN
    my $Setup = $self->_setupPrettyURLS({
        IsUnityRegistered      => 1,
        IsPortalSiteDeprecated => undef,
        ExportAllProducts      => 1,
    });
    my $SUT = $Setup->{'SUT'};
    my $Servlet = $Setup->{'Servlet'};
    my $PortalConfig = $Setup->{'PortalConfig'};

    #WHEN
    $SUT->GeneratePortalConfigProductPrettyUnityUrls($Servlet);

    #THEN
    is(GetCallCount($SUT, '_GetAndSaveProductPrettyUnityUrls'), 0, "$SubTestName - Portal config pretty url request generation does not get called with 0 products.");

    my $PortalConfigFromServlet = $Servlet->object();
    my $ShopFromServlet = $PortalConfigFromServlet->getSite();

    my @Products = ({'id' => 1});
    $SUT = $self->_createSUT(\@Products);

    $SUT->GeneratePortalConfigProductPrettyUnityUrls($Servlet);
    is(GetCallCount($SUT, '_GetAndSaveProductPrettyUnityUrls'), 1, "$SubTestName - Portal config pretty url request generation gets called 1 time with 1 product.");

    is_deeply(
        GetParametersFromMockifyCall($SUT, '_GetAndSaveProductPrettyUnityUrls', 0),
        [$ShopFromServlet, $PortalConfig, [{'id' => 1}]],
        "$SubTestName - Portal config pretty url request generation on first call has the correct parameters."
    );

    # Due to Json overhead we have a 100089 ByteLength String with 841 products / 1681 = 840 * 2 + 1"
    my $StringWith100Bytes = '30mRDaVBsisUH1VorSRt2PzGH6vVgylUWMkLogj8K5vWvq4TBW0KKKZdqL8vDzgUbXddbuzSVzTutIUPwKjBhze2TYRCQiKgaANR';
    @Products = map { {'name' => "$StringWith100Bytes -> $_"} } (0 .. 1681);
    # is(length(encode('UTF-8',to_json(\@Products))), 200731, "JSON ByteLength of Products is 200731");

    $SUT = $self->_createSUT(\@Products);

    $SUT->GeneratePortalConfigProductPrettyUnityUrls($Servlet);
    is(GetCallCount($SUT, '_GetAndSaveProductPrettyUnityUrls'), 3, "$SubTestName - Portal config pretty url request generation gets called 3 times with 1681 products.");

    my $ResultThirdCall = GetParametersFromMockifyCall($SUT, '_GetAndSaveProductPrettyUnityUrls', 2);

    my @ExpectedResultThirdCall = map { {'name' => "$StringWith100Bytes -> $_"} } (1675 .. 1681);
    is_deeply(
        $ResultThirdCall,
        [$ShopFromServlet, $PortalConfig, \@ExpectedResultThirdCall],
        "$SubTestName - Portal config pretty url request generation on third call has the correct parameters with 1681 products."
    );
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($Products) = @_;

    my $aParameterList = [];
    my $Mockify = Test::Mockify::Sut->new( 'DE_EPAGES::ProductPortal::UI::PortalConfig', $aParameterList );

    $Mockify->spy('_createPortalConfig')
        ->when(
            Object('DE_EPAGES::Presentation::Servlet::Permission'),
            Object('DE_EPAGES::ProductPortal::API::Object::PortalSite'),
            Object('DE_EPAGES::Object::API::Object::Object'),
            Object('DE_EPAGES::Object::API::Object::Class'));

    $Mockify->mock('NewObject')
        ->when(
            Object('DE_EPAGES::Presentation::Servlet::Permission'),
            Object('DE_EPAGES::Object::API::Object::Object'),
            Object('DE_EPAGES::Object::API::Object::Class'),
            HashRef()
        )->thenReturnUndef();
    $Mockify->mock('_removeExportFile')
        ->when(Object('DE_EPAGES::ProductPortal::API::Object::PortalConfig'))
        ->thenReturnUndef();
    $Mockify->spy('_GetAndSaveProductPrettyUnityUrls')
        ->when(
            Object('DE_EPAGES::Shop::API::Object::Shop'),
            Object('DE_EPAGES::ProductPortal::API::Object::PortalConfig'),
            ArrayRef()
        );

    $Mockify->mockImported('DE_EPAGES::Product::API::Product', 'GetProductPrettyUnityUrlsRequest')
        ->whenAny()
        ->thenReturn($Products);

    $Mockify->mockImported('DE_EPAGES::Product::API::Product', 'SaveProductPrettyUrlsResponse')
        ->whenAny()
        ->thenReturn({});

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($hParams) = @_;


    my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new( $hParams->{'Shop'} // $hParams->{'PortalConfig'} );
    $Mockify->mock('form',$hParams->{'Form'});
    $Mockify->mock('user',$hParams->{'User'});
    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($hParams) = @_;

    my $aPortalConfigs = ($hParams->{'DeletePortalConfig'}) ?  $hParams->{'PortalConfigs'} : [];
    my $hAttributes = {
        'Alias' => 'MyShop',
        'IsUnityRegistered' => $hParams->{'IsUnityRegistered'},
        'PortalConfigs' => $aPortalConfigs,
        'LanguageID' => 1,
        'LocaleID' => 'de_DE'
    };

    my $PortalSite = $hParams->{'PortalSite'};
    my $folder = $hParams->{'Folder'};

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop',$hAttributes);

    my $aParameterTypes = [ {'string' => 'POST'}, {'string' => '/products/get-urls'}, 'hashref', 'arrayref', 'arrayref' ];
    $Mockify->addMockWithReturnValueAndParameterCheck('sendUnityRequest', [], $aParameterTypes);

    $aParameterTypes = [ 'int', {'string' => 'Object'}, 'hashref' ];
    $Mockify->addMockWithReturnValueAndParameterCheck('loadObject', $PortalSite, $aParameterTypes);

    $aParameterTypes = [ {'string' => 'PortalConfigs'}, 'hashref' ];
    $Mockify->addMockWithReturnValueAndParameterCheck('folder', $folder, $aParameterTypes);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPortalConfig {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'CountProducts'     => 0,
        'ExportAllProducts' => $hParams->{'ExportAllProducts'},
        'LanguageID'        => 1,
        'Portal'            => $hParams->{'Portal'},
        'Site'              => $hParams->{'Shop'},
        'ID'                => $hParams->{'ID'},
        'Alias'             => $hParams->{'Alias'},
        'PortalSite'        => $hParams->{'PortalSite'},
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalConfig',
        $hAttributes
    );

    $Mockify->addMockWithReturnValue('items', []);
    $Mockify->addMock('delete', sub {return;} );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;
    my ($hParams) = @_;

    my $aParameterList = [];
    my $Mockify = Test::Mockify->new(
                   'DE_EPAGES::Presentation::API::Form',
                   $aParameterList
              );
    my @ListOfPortalConfigsAndSites;
    push(@ListOfPortalConfigsAndSites, EXISTING_PORTAL_CONFIG_ID) if !($hParams->{'DeletePortalConfig'});
    push(@ListOfPortalConfigsAndSites, PORTAL_SITE_ID) if ($hParams->{'AddPortalConfig'});
    if (scalar(@ListOfPortalConfigsAndSites) > 0) {
        $Mockify
            ->mock('value')
            ->when( String('PortalID') )->thenReturn(@ListOfPortalConfigsAndSites);
    } else {
        $Mockify
            ->mock('value')
            ->when( String('PortalID') )->thenReturnUndef();
    }
    $Mockify
        ->mock('value')
        ->when( String('ACCEPT') )->thenReturn($hParams->{'ACCEPT'} // 0);

    if (defined $hParams->{'OnlyAmazon'}) {
        $Mockify
            ->mock('value')
            ->when( String('OnlyAmazon'))
            ->thenReturn($hParams->{'OnlyAmazon'});
        $Mockify
            ->mock('formValues')
            ->whenAny()
            ->thenReturn({'OnlyAmazon' => $hParams->{'OnlyAmazon'}});
    } else {
        $Mockify
            ->mock('value')
            ->when( String('OnlyAmazon'))
            ->thenReturnUndef();
    }
    if (defined $hParams->{'OnlyGoogleShopping'}) {
        $Mockify
            ->mock('value')
            ->when( String('OnlyGoogleShopping'))
            ->thenReturn($hParams->{'OnlyGoogleShopping'});
        $Mockify
            ->mock('formValues')
            ->whenAny()
            ->thenReturn({'OnlyGoogleShopping' => $hParams->{'OnlyGoogleShopping'}});
    } else {
        $Mockify
            ->mock('value')
            ->when( String('OnlyGoogleShopping'))
            ->thenReturnUndef();
    }
    if (not defined $hParams->{'OnlyAmazon'} and not defined $hParams->{'OnlyGoogleShopping'}) {
        $Mockify
            ->mock('formValues')
            ->whenAny()
            ->thenReturn({});
    }

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPortalSite {
    my $self = shift;
    my ($hParams) = @_;

    my $DateInThePast = SubtractDuration(GetCurrentDateTime(), 'months', 1);
    my $IsDeprecated = $hParams->{'IsPortalSiteDeprecated'};
    my $DeprecationDate = $IsDeprecated ? $DateInThePast : undef;
    my $hAttributes = {
        'Alias' => $hParams->{'Alias'} // 'myPortalSite',
        'DeprecationDate' => $DeprecationDate,
        'ID' => PORTAL_SITE_ID,
        'instanceOf' => 'PortalSite',
        'LocaleID' => 'de_DE',
        'Portal' => $self->_createPortal,
        'PortalConfigClass' => $self->_createPortalConfigClass()
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalSite',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPortal {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'PortalConfigClass' => $self->_createPortalConfigClass(),
        'Alias' => $hParams->{'Alias'}
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::Portal',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createFolderObject {
    my $self = shift;

    my $hParameters = {
        'Child' => {}
    };
    my $EpagesMockObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Object', $hParameters);

    return $EpagesMockObject;
}

#----------------------------------------------------------------------------------------
sub _createPortalConfigClass {
    my $self = shift;

    my $hAttributes = {};
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', $hAttributes);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createUser {
    my $self = shift;

    my $hAttributes = {
        'DeleteConfirmation' => 1
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object', $hAttributes);

    return $Mockify->getMockObject();
}

sub _setupUpdatePortalConfig {
    my $self = shift;
    my ($hParams) = @_;
    $hParams->{ 'ShopInServlet' } = 1;

    return $self->_setupTestConditions($hParams);
}

sub _setupPrettyURLS {
    my $self = shift;
    my ($hParams) = @_;
    $hParams->{'ShopInPortalConfig'} = 1;

    return $self->_setupTestConditions($hParams);
}

sub _setupTestConditions {
    my $self = shift;
    my ($hParams) = @_;

    my $ObjectCreationParams = $self->_prepareObjectParameters($hParams);

    my $hShopParams = $ObjectCreationParams->{'ShopParams'};
    my $hPortalConfigParams = $ObjectCreationParams->{'PortalConfigParams'};
    my $hServletParams = $ObjectCreationParams->{'ServletParams'};

    my $PortalConfig;
    my $Shop;
    if ($hParams->{'ShopInServlet'}) {
        my $aPortalConfigs = [];
        my $ID = EXISTING_PORTAL_CONFIG_ID;
        my $aExistingPortalAliases  = $hParams->{'ExistingPortalAliases'} // ['myPortal'];

        foreach my $ExistingAlias (@$aExistingPortalAliases) {
            my $hPortalParams = {
                Alias => $ExistingAlias
            };
            my $Portal = $self->_createPortal($hPortalParams);

            $hPortalConfigParams = {
                Portal => $Portal,
                ID     => $ID,
                Alias  => $ExistingAlias,
            };
            $PortalConfig = $self->_createPortalConfig($hPortalConfigParams);
            push(@$aPortalConfigs, $PortalConfig);
            $ID += 1;
        }

        $hShopParams->{'PortalConfigs'} = $aPortalConfigs;
        $Shop = $self->_createShop($hShopParams);

        $hServletParams->{'Shop'} = $Shop;
    }
    elsif ($hParams->{'ShopInPortalConfig'}) {
        $Shop = $self->_createShop($hShopParams);
        $hPortalConfigParams->{'Shop'} = $Shop;
        $PortalConfig = $self->_createPortalConfig($hPortalConfigParams);

        $hServletParams->{ PortalConfig } = $PortalConfig;
    }

    my $Servlet = $self->_createServlet($hServletParams);
    my $Products = [];
    my $SUT = $self->_createSUT($Products);

    return {
        SUT           => $SUT,
        Servlet       => $Servlet,
        PortalConfig => $PortalConfig,
    }
}

sub _prepareObjectParameters {
    my $self = shift;
    my ($hParams) = @_;

    my $Portal = $self->_createPortal({
        Alias => "myPortal"
    });

    my $PortalSite = $self->_createPortalSite({
        Alias                  => 'myPortalSite',
        IsPortalSiteDeprecated => $hParams->{'IsPortalSiteDeprecated'},
    });

    my $Folder = $self->_createFolderObject();

    my $Form = $self->_createForm($hParams);

    my $User = $self->_createUser();

    my $hShopParams = {
        DeletePortalConfig => $hParams->{'DeletePortalConfig'},
        PortalConfigs      => [],
        PortalSite         => $PortalSite,
        IsUnityRegistered  => $hParams->{'IsUnityRegistered'},
        Folder             => $Folder,
    };

    my $hPortalConfigParams = {
        ExportAllProducts => $hParams->{'ExportAllProducts'},
        IsUnityRegistered => $hParams->{'IsUnityRegistered'},
        Portal            => $Portal,
        Shop              => undef,
        PortalSite        => $PortalSite
    };

    my $hServletParams = {
        Form         => $Form,
        User         => $User,
        PortalConfig => undef,
        Shop         => undef
    };

    my $hSutParams = {
        Folder   => $Folder,
        Products => []
    };

    return {
        ShopParams         => $hShopParams,
        PortalConfigParams => $hPortalConfigParams,
        ServletParams      => $hServletParams,
        SUTParams          => $hSutParams,
    }
}

__PACKAGE__->runScript();
