#========================================================================================
# package      DE_EPAGES::Saferpay2::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Saferpay2::API::Cartridge;
use base DE_EPAGES::Payment::API::PaymentInstaller;

use strict;



#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Saferpay2::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Saferpay2.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Saferpay2',
        'Version'            => '7.75.0',
        'Patches'            => ['7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0'],
    );

    return bless $self, $class;
}



#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall();
    $self->SUPER::install();
    $self->finishInstall();

    return;
}



#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall();
    $self->SUPER::uninstall();
    $self->finishUninstall();

    return;
}

1;