#==============================================================================
# package      DE_EPAGES::ServiRed::Hooks::OrderProcess
# state        private
#==============================================================================
package DE_EPAGES::ServiRed::Hooks::OrderProcess;

use strict;
use DE_EPAGES::ExternalPayment::API::Constants qw (DEFAULT_PAYMENT_STEP_POSITION);

#========================================================================================
# function     OnRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# description  register payment step for checkout process
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object;
    my $Container = $BasketOrder->container;
    return unless defined $Container;

    my $pli = $Container->get('Payment');
    return unless defined($pli);

    if ($pli->instanceOf('LineItemPaymentServiRed')) {
        $BasketOrderProcess->addStep('ServiRed', DEFAULT_PAYMENT_STEP_POSITION);
    } elsif ($pli->instanceOf('LineItemPaymentBizum')) {
        $BasketOrderProcess->addStep('Bizum', DEFAULT_PAYMENT_STEP_POSITION);
    }
    return;
}

#========================================================================================
# function     AfterRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# description  set order step; remove payment step if necessary
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object;
    my $Container = $BasketOrder->container;
    return unless defined $Container;

    my $pli = $Container->get('Payment');
    return unless defined $pli;

    if ($pli->instanceOf('LineItemPaymentServiRed')) {
        if ($pli->get('PaymentMethod')->get('DoPaymentOnBasket')) {
            $BasketOrderProcess->setOrderStep('ServiRed');
        }
        $BasketOrderProcess->deleteStep('Bizum');
    }
    elsif ($pli->instanceOf('LineItemPaymentBizum')) {
        if ($pli->get('PaymentMethod')->get('DoPaymentOnBasket')) {
            $BasketOrderProcess->setOrderStep('Bizum');
        }
        $BasketOrderProcess->deleteStep('ServiRed');
    }
    else {
        $BasketOrderProcess->deleteStep('ServiRed');
        $BasketOrderProcess->deleteStep('Bizum');
    }

    return;
}

1;
