#========================================================================================
# package      DE_EPAGES::Shipping::t::AvailableShippingFulfillmentTypes
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit Test for class AvailableShippingFulfillmentTypes
#----------------------------------------------------------------------------------------
#========================================================================================
package DE_EPAGES::Shipping::t::AvailableShippingFulfillmentTypes;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use strict;
use Test::MockObject::Extends;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Shipping::API::TLE::AvailableShippingFulfillmentTypesHandler;
use Test::Mockify;

## no critic qw(ProhibitLongLines)

sub test {
    my $self = shift;
    $self->test_getAvailableShippingFulfillmentTypesWithProviderType();
    $self->test_getAvailableShippingFulfillmentTypesWithoutProviderType();
    $self->test_getAvailableShippingFulfillmentTypesWithEmptyShop();
    $self->test_getAvailableShippingFulfillmentTypesWithAndWithoutProviderType();
    $self->test_getAvailableShippingFulfillmentTypesWithAndWithoutProviderType_deprecated_unity();
    $self->test_getAvailableShippingFulfillmentTypesWithAndWithoutProviderType_deprecated_base();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_getAvailableShippingFulfillmentTypesWithProviderType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $System = $self->_createSystemObjectWithProviderType();
    my $AvailableShippingFulfillmentTypesHandlerMocked = $self->_getAvailableShippingFulfillmentTypesHandlerMocked($System);

    my $Shop = $self->getMockedShop();

    my $aShippingObjectTypes = $AvailableShippingFulfillmentTypesHandlerMocked->_getAvailableShippingFulfillmentTypes($Shop); ## no critic qw(ProhibitPrivateSubroutineUse)

    is(scalar @$aShippingObjectTypes, 1, "$SubTestName - The number of the available ShippingFulfillmentTypes returned must be 1.");
    is($aShippingObjectTypes->[0]->{'ShippingFulfillmentType'}->{'MarketingText'},
        'this is a ShippingProvider from a ShippingFulfillmentType',
        "$SubTestName - Marketing text for the shipping fulfillment provider.");
    is($aShippingObjectTypes->[0]->{'ShippingFulfillmentType'}->{'Name'},
        'ShippingFulfillmentType',
        "$SubTestName - Name for the shipping fulfillment provider.");
    is(ref($aShippingObjectTypes->[0]->{'ShippingFulfillmentType'}->{'ShippingProviderTypes'}),
        'ARRAY',
        "$SubTestName - The field ShippingProviderTypes of a ShippingFulfillmentType is an array.");
    is($aShippingObjectTypes->[0]->{'ShippingProviderType'}->{'IsVisible'},
        1,
        "$SubTestName - IsVisible attribute for the first provider should be 1.");
    is($aShippingObjectTypes->[0]->{'ShippingProviderType'}->{'MarketingText'},
        'this is royalmail from Shippo',
        "$SubTestName - Marketing text for the first provider type is 'this is royalmail from Shippo'.");
    is($aShippingObjectTypes->[0]->{'ShippingProviderType'}->{'Name'},
        'RoyalMail(Shippo)',
        "$SubTestName - Name text for the first provider type is 'RoyalMail(Shippo)'.");
}

#----------------------------------------------------------------------------------------
sub test_getAvailableShippingFulfillmentTypesWithoutProviderType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $System = $self->_createSystemObjectWithoutProviderType();
    my $AvailableShippingFulfillmentTypesHandlerMocked = $self->_getAvailableShippingFulfillmentTypesHandlerMocked($System);

    my $Shop = $self->getMockedShop();

    my $aShippingObjectTypes = $AvailableShippingFulfillmentTypesHandlerMocked->_getAvailableShippingFulfillmentTypes($Shop); ## no critic qw(ProhibitPrivateSubroutineUse)

    is(scalar @$aShippingObjectTypes, 1, "$SubTestName - The number of the available ShippingFulfillmentTypes returned must be 0.");
    is($aShippingObjectTypes->[0]->{'ShippingFulfillmentType'}->{'MarketingText'},
        'this is a ShippingProvider from a ShippingFulfillmentType',
        "$SubTestName - Marketing text for the first provider type is 'this is a ShippingProvider'");
    is($aShippingObjectTypes->[0]->{'ShippingFulfillmentType'}->{'Name'},
        'ShippingFulfillmentType',
        "$SubTestName - Name text for the first provider type is 'ShippingProvider'");
}

#----------------------------------------------------------------------------------------
sub test_getAvailableShippingFulfillmentTypesWithEmptyShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $System = $self->_createSystemObjectWithoutProviderType();
    my $AvailableShippingFulfillmentTypesHandlerMocked = $self->_getAvailableShippingFulfillmentTypesHandlerMocked($System);

    my $Shop;

    my $aShippingObjectTypes = $AvailableShippingFulfillmentTypesHandlerMocked->_getAvailableShippingFulfillmentTypes($Shop); ## no critic qw(ProhibitPrivateSubroutineUse)

    is_deeply($aShippingObjectTypes,[],"$SubTestName - The call with an empty Shop should be an empty array'");
}

#----------------------------------------------------------------------------------------
sub test_getAvailableShippingFulfillmentTypesWithAndWithoutProviderType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $System = $self->_createSystemObjectWithAndWithoutProviderType();
    my $AvailableShippingFulfillmentTypesHandlerMocked = $self->_getAvailableShippingFulfillmentTypesHandlerMocked($System);

    my $Shop = $self->getMockedShop();

    my $aShippingObjectTypes = $AvailableShippingFulfillmentTypesHandlerMocked->_getAvailableShippingFulfillmentTypes($Shop); ## no critic qw(ProhibitPrivateSubroutineUse)

    is(scalar @$aShippingObjectTypes, 2, "$SubTestName - The number of ShippingFulfillmentTypes returned must be 2.");
    is($aShippingObjectTypes->[0]->{'ShippingProviderType'}->{'IsVisible'},
        1,
        "$SubTestName - IsVisible attribute for the first provider should be 1.");
    is($aShippingObjectTypes->[0]->{'ShippingProviderType'}->{'MarketingText'},
        'this is royalmail from Shippo',
        "$SubTestName - Marketing text for the first provider type is 'this is royalmail from Shippo'");
    is($aShippingObjectTypes->[0]->{'ShippingProviderType'}->{'Name'},
        'RoyalMail(Shippo)',
        "$SubTestName - Name text for the first provider type is 'RoyalMail(Shippo)'");
    is($aShippingObjectTypes->[1]->{'ShippingProviderType'}->{'MarketingText'},
        'this is a ShippingProvider from a ShippingFulfillmentType',
        "$SubTestName - Marketing text for the first provider type is 'this is a ShippingProvider from a ShippingFulfillmentType'");
    is($aShippingObjectTypes->[1]->{'ShippingProviderType'}->{'Name'},
        'ShippingFulfillmentType',
        "$SubTestName - Name text for the first provider type is 'ShippingProvider'");
}

#----------------------------------------------------------------------------------------
sub test_getAvailableShippingFulfillmentTypesWithAndWithoutProviderType_deprecated_unity {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $IsDeprecated = {
        'Base'              => 0,
        'Unity'             => 1,
        'ShopType'          => 'Unity',
    };
    my $System = $self->_createSystemObjectWithAndWithoutProviderType($IsDeprecated);
    my $AvailableShippingFulfillmentTypesHandlerMocked = $self->_getAvailableShippingFulfillmentTypesHandlerMocked($System);
    my $isUnityRegistered = 1;
    my $Shop = $self->getMockedShop($isUnityRegistered);

    my $aShippingObjectTypes = $AvailableShippingFulfillmentTypesHandlerMocked->_getAvailableShippingFulfillmentTypes($Shop); ## no critic qw(ProhibitPrivateSubroutineUse)

    is(scalar @$aShippingObjectTypes, 0, "$SubTestName - The number of ShippingFulfillmentTypes returned must be 0.");
}

#----------------------------------------------------------------------------------------
sub test_getAvailableShippingFulfillmentTypesWithAndWithoutProviderType_deprecated_base {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $IsDeprecated = {
        'Base'  => 1,
        'Unity' => 0,
        'ShopType' => 'Base',

    };
    my $System = $self->_createSystemObjectWithAndWithoutProviderType($IsDeprecated);
    my $AvailableShippingFulfillmentTypesHandlerMocked = $self->_getAvailableShippingFulfillmentTypesHandlerMocked($System);
    my $Shop = $self->getMockedShop();

    my $aShippingObjectTypes = $AvailableShippingFulfillmentTypesHandlerMocked->_getAvailableShippingFulfillmentTypes($Shop); ## no critic qw(ProhibitPrivateSubroutineUse)

    is(scalar @$aShippingObjectTypes, 0, "$SubTestName - The number of ShippingFulfillmentTypes returned must be 0.");
}


#----------------------------------------------------------------------------------------
sub _getAvailableShippingFulfillmentTypesHandlerMocked{
    my $self = shift;
    my ($System) = @_;

    my $AvailableShippingFulfillmentTypesHandler = DE_EPAGES::Shipping::API::TLE::AvailableShippingFulfillmentTypesHandler->new();
    my $AvailableShippingFulfillmentTypesHandlerMocked = Test::MockObject::Extends->new($AvailableShippingFulfillmentTypesHandler);
    $AvailableShippingFulfillmentTypesHandlerMocked->{'System'} = $System;

    $AvailableShippingFulfillmentTypesHandlerMocked->mock('_LoadRootObject', sub {
        shift;
        return $AvailableShippingFulfillmentTypesHandlerMocked->{'System'};
    });

    $AvailableShippingFulfillmentTypesHandlerMocked->mock('_IsFeatureAvailable', sub {
       shift;
       return 'Visible';
    });

    return $AvailableShippingFulfillmentTypesHandlerMocked;
}

#----------------------------------------------------------------------------------------
sub _createSystemObjectWithProviderType{
    my $self = shift;
    my $ShippingProviderTypes = $self->getShippingProviderTypes();
    my $ShippingFulfillmentTypes = $self->getShippingFulfillmentTypes($ShippingProviderTypes);
    my $Attributes = {
        'ShippingFulfillmentTypes' => $ShippingFulfillmentTypes,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _createSystemObjectWithoutProviderType{
    my $self = shift;
    my $emptyArray = [];
    my $ShippingFulfillmentTypes = $self->getShippingFulfillmentTypes($emptyArray);
    my $Attributes = {
        'ShippingFulfillmentTypes' => $ShippingFulfillmentTypes,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _createSystemObjectWithAndWithoutProviderType{
    my $self = shift;
    my ($hIsDeprecated) = @_;
    my $emptyArray = [];
    my $ShippingProviderTypes = $self->getShippingProviderTypes();
    my $ShippingFulfillmentTypesWithProviderTypes = $self->getShippingFulfillmentTypes($ShippingProviderTypes, $hIsDeprecated);
    my $ShippingFulfillmentTypesWithoutProviderTypes = $self->getShippingFulfillmentTypes($emptyArray, $hIsDeprecated);
    my @ShippingFulfillmentTypes = ( @$ShippingFulfillmentTypesWithProviderTypes , @$ShippingFulfillmentTypesWithoutProviderTypes);
    my $Attributes = {
        'ShippingFulfillmentTypes' => \@ShippingFulfillmentTypes,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site',$Attributes);
}

#----------------------------------------------------------------------------------------
sub getMockedShop {
    my $self = shift;
    my @emptyArray;
    my $Attributes = {
        'ShippingFulfillmentMethods' => \@emptyArray,
        'ShippingProviderMethods'    => \@emptyArray
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $Attributes);
}

#----------------------------------------------------------------------------------------
sub getShippingFulfillmentTypes {
    my $self = shift;
    my ($ShippingProviderTypes, $hIsDeprecated) = @_;

    my $Attributes = {
        'ID'                    => 12345,
        'ShippingProviderTypes' => $ShippingProviderTypes,
        'MethodClass'           => '',
        'Name'                  => 'ShippingFulfillmentType',
        'MarketingText'         => 'this is a ShippingProvider from a ShippingFulfillmentType',
        'ProviderMethodClass'   => ''
    };
    my $MockedShippingFulfillmentTypes =
        DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType', $Attributes);
    if ($hIsDeprecated->{'ShopType'} eq 'Unity') {
        $MockedShippingFulfillmentTypes->mock(
            'isDeprecated',
            sub {shift;
                return $hIsDeprecated->{'Unity'};
            }
        );
    } elsif($hIsDeprecated->{'ShopType'} eq 'Base') {
        $MockedShippingFulfillmentTypes->mock(
            'isDeprecated',
            sub {shift;
                return $hIsDeprecated->{'Base'};
            }
        );
    } else {
        $MockedShippingFulfillmentTypes->mock(
            'isDeprecated',
            sub {shift;
                return 0;
            }
        );
    }

    return [$MockedShippingFulfillmentTypes];
}

#----------------------------------------------------------------------------------------
sub getShippingProviderTypes {
    my $self = shift;
    my $Result = [] ;
    my $AttributesRoyal = {
        'Name'          => 'RoyalMail(Shippo)',
        'MarketingText' => 'this is royalmail from Shippo',
        'IsVisible'     => '1'
    };

    push(
         @$Result,
         DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::ExternalShipping::API::Object::ShippingProviderType', $AttributesRoyal)
    );

    my $AttributesDHL = {
        'Name'     => 'DHL(Shippo)',
        'MarketingText' => 'this is DHL from Shippo',
        'IsVisible'     => '0'
    };

    push(
         @$Result,
         DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::ExternalShipping::API::Object::ShippingProviderType', $AttributesDHL)
    );

    return $Result;
}

__PACKAGE__->runScript();